# -*- coding: utf8 -*-

from __future__ import unicode_literals

import datetime

parser_class = "acltop"


def test_resulat(self):
    date = datetime.datetime(2013, 3, 27, 14, 17, 49)
    result = self.decode(
        """
    H|\\^&|<1364389246_15004><1364390269_15509>||ACL TOP|||||LIS||P|1394-97|20130327141749
    P|1||030314230792||TEST^ANONYME||19260703|M
    O|1|6986686803|<1364389244_14980>|^^^501|R|20130327084000|||||||||P||||||||||O\\F
    R|1|^^^501|21.0|s||N||F\\V||mub^mub||20130327141749|ACL TOP^03^6
    R|2|^^^501|42|%||N||F\\V||mub^mub||20130327141749|ACL TOP^03^6
    C|1|I|5507^Inférieur à la limite normal^LN|I
    C|1|I|1030^cuvette shuttle temp warning^HW|I
    C|1|I|5525^reagent temperature warning^LN|I
    R|3|^^^501|1.88|INR||N||F\\V||mub^mub||20130327141749|ACL TOP^03^6
    R|4|^^^508|Echec|g/L||N||X\\V||mub^mub||20130327141749|ACL TOP^03^6
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "6986686803",
                "type": "patient",
                "results": [
                    {
                        "date": date,
                        "value": "21.0",
                        "code": "501s",
                        "flags": ["N"],
                        "operator": "mub",
                    },
                    {
                        "date": date,
                        "value": "42",
                        "code": "501%",
                        "flags": ["1030", "5507", "5525", "HW", "LN", "N"],
                        "operator": "mub",
                    },
                    {
                        "date": date,
                        "value": "1.88",
                        "code": "501INR",
                        "flags": ["N"],
                        "operator": "mub",
                    },
                    {
                        "date": date,
                        "value": "Echec",
                        "code": "508g/L",
                        "flags": ["N"],
                        "operator": "mub",
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_result_TP_with_reactif(self):
    date = datetime.datetime(2018, 1, 11, 13, 26, 36)
    date_expiration = datetime.datetime(2018, 8, 1, 0, 0)
    result = self.decode(
        """
    H|@^\\|F1EDDB49B719496F9779D8EDC6842B37||ACL TOP|||||LIS||P|1394-97|20180111132637
    P|1||1234||TEST^Anonyme||19901104|M|||||^
    O|1|123456|<1515673421_20338>|^^^131|R|20180111114705|||||||||P||||||||||O@F
    R|1|^^^131|32.9|s||N||F@V||hs||20180111132636|ACL TOP^58^3
    C|1|I|LI-TP ReadiPlasTin^N0864563^2018-8|I
    R|2|^^^131|26|%||N||F@V||hs||20180111132636|ACL TOP^58^3
    R|3|^^^131|2.86|INR||N||F@V||hs||20180111132636|ACL TOP^58^3
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "123456",
                "type": "patient",
                "results": [
                    {
                        "date": date,
                        "value": "32.9",
                        "code": "131s",
                        "flags": ["N"],
                        "operator": "hs",
                        "lots": [
                            {
                                "lot": "TP ReadiPlasTinN0864563",
                                "expiration": date_expiration,
                                "type": "reagent",
                            }
                        ],
                    },
                    {
                        "date": date,
                        "value": "26",
                        "code": "131%",
                        "flags": ["N"],
                        "operator": "hs",
                        "lots": [
                            {
                                "lot": "TP ReadiPlasTinN0864563",
                                "expiration": date_expiration,
                                "type": "reagent",
                            }
                        ],
                    },
                    {
                        "date": date,
                        "value": "2.86",
                        "code": "131INR",
                        "flags": ["N"],
                        "operator": "hs",
                        "lots": [
                            {
                                "lot": "TP ReadiPlasTinN0864563",
                                "expiration": date_expiration,
                                "type": "reagent",
                            }
                        ],
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_resultat_lots_reactifs(self):
    result = self.decode(
        """
    H|\\^&|<1364389246_15004><1364390269_15509>||ACL TOP|||||LIS||P|1394-97|20130327141749
    P|1||030314230792||TEST^ANONYME||19260703|M
    O|1|6986686803|<1364389244_14980>|^^^501|R|20130327084000|||||||||P||||||||||O\\F
    R|1|^^^501|21.0|s||N||F\\V||mub^mub||20130327141749|ACL TOP^03^6
    C|1|I|5801^Software in experimental operation and/or Service^IU|I
    C|2|I|LI-PS DEF PLASMA^LOT-QQ2-3^2018-2|I
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "6986686803",
                "type": "patient",
                "results": [
                    {
                        "code": "501s",
                        "value": "21.0",
                        "date": datetime.datetime(2013, 3, 27, 14, 17, 49),
                        "flags": ["5801", "IU", "N"],
                        "lots": [
                            {
                                "lot": "PS DEF PLASMALOT-QQ2-3",
                                "expiration": datetime.datetime(2018, 2, 1, 0, 0),
                                "type": "reagent",
                            }
                        ],
                        "operator": "mub",
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_qc(self):
    result = self.decode(
        """
    H|\\^&|<1364388003_16777262><1364388203_14472>||ACL TOP|||||LIS||P|1394-97|20130327134323
    P|1||||^|||U|||||^
    O|1|CQ Anormal Bas||^^^501|R|20130327134004|||||Q||||P||||||||||O\\F
    R|1|^^^501|34|%||N||F\\V||mub^mub||20130327134323|ACL TOP^DA^4
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "CQ Anormal Bas",
                "type": "control",
                "control_level": None,
                "control_name": None,
                "control_lot": "CQ Anormal Bas",
                "results": [
                    {
                        "code": "501%",
                        "date": datetime.datetime(2013, 3, 27, 13, 43, 23),
                        "value": "34",
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_order(self):
    result = self.order()
    expected = """H|\\^&|||LIS|||||ACL TOP||P|1394-97|20130715141912
P|1||036666666||NYME^ANO||20110513|F|||||LOC
O|1|123456||^^^A|S|20130513114242|||||A||||P||||||||||O
O|2|123456||^^^B|S|20130513114242|||||A||||P||||||||||O
O|3|123456||^^^C|S|20130513114242|||||C||||P||||||||||O
L|1|N"""
    assert result == expected


def test_query(self):
    result = self.decode(
        """H|\\^&|||ACL TOP|||||LIS||P|1394-97|20130925092455
        Q|1|^2468404903||||||||||O\\N
        L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "2468404903"}],
    }
    assert result == expected


def test_void_order(self):
    result = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """H|\\^&|||ACL TOP|||||LIS||P|1394-97|20130925092455
                    Q|1|^2468404903||||||||||O\\N
                    L|1|N"""
            ),
            "control_id": "20130715141912",
        },
    )
    expected = """
    H|\\^&|||LIS|||||ACL TOP||P|1394-97|20130715141912
    L|1|I"""
    assert result == expected


def test_multiquery(self):
    result = self.decode(
        """
    H|\\^&|<1382439495_1205><1382439495_1206>||ACL TOP|||||LIS||P|1394-97|20131022125815
    Q|1|^2195114903\\^6995683503\\^6995683603\\^5095505703\\^2395307203\\^2295203503\\^2195113903\\^3195354003\\^3195352703\\^3195356303||||||||||O\\N
    L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [
            {"type": "patient", "sample_id": "2195114903"},
            {"type": "patient", "sample_id": "6995683503"},
            {"type": "patient", "sample_id": "6995683603"},
            {"type": "patient", "sample_id": "5095505703"},
            {"type": "patient", "sample_id": "2395307203"},
            {"type": "patient", "sample_id": "2295203503"},
            {"type": "patient", "sample_id": "2195113903"},
            {"type": "patient", "sample_id": "3195354003"},
            {"type": "patient", "sample_id": "3195352703"},
            {"type": "patient", "sample_id": "3195356303"},
        ],
    }
    assert result == expected


def test_qc_prefix(self):
    date = datetime.datetime(2013, 10, 23, 12, 26, 37)
    self.configure(qc_prefix="CQ", qc_5653_prefix=True, qc_inclusion_alarmes=True)
    result = self.decode(
        """
    H|\\^&|<1382523618_3998><1382523997_4049>||ACL TOP|||||LIS||P|1394-97|20131023122637
    P|1||||^|||U|||||^
    O|1|CQHM04_008Y13|<1382523606_3989>|^^^510|R|20131023122018|||||||||P||||||||||O\\F
    R|1|^^^510|42.9|s||N||F\\V||flo^flo||20131023122637|ACL TOP^62^1
    C|1|I|5600^Maintenance dépassée ou en échec^MT|I
    R|2|^^^510|1.38|Ratio||N||F\\V||flo^flo||20131023122637|ACL TOP^62^1
    C|1|I|5600^Maintenance dépassée ou en échec^MT|I
    O|2|CQHM04_008Y13|<1382523606_3989>|^^^510|R|20131023122018|||||||||P||||||||||O\\F
    R|1|^^^510|42.9|s||N||F\\V||flo^flo||20131023122637|ACL TOP^62^1
    C|1|I|5653^Lot alternatif en cours d'utilisation^ME|I
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "CQHM04_008Y13",
                "control_lot": "CQHM04_008Y13",
                "control_name": None,
                "control_level": None,
                "results": [
                    {
                        "date": date,
                        "value": "42.9",
                        "flags": ["5600", "MT", "N"],
                        "code": "510s",
                    },
                    {
                        "date": date,
                        "value": "1.38",
                        "code": "510Ratio",
                        "flags": ["5600", "MT", "N"],
                    },
                ],
            },
            {
                "type": "control",
                "sample_id": "QUALIF-CQHM04_008Y13",
                "control_lot": "QUALIF-CQHM04_008Y13",
                "control_name": None,
                "control_level": None,
                "results": [
                    {
                        "date": date,
                        "value": "42.9",
                        "code": "510s",
                        "flags": ["5653", "ME", "N"],
                    }
                ],
            },
        ],
    }
    assert result == expected


def test_qc_new_comportement(self):
    self.configure(qc_new_comportement=True, qc_inclusion_alarmes=True)
    result = self.decode(
        """
    H|@^\\|748E54D85F354FDF9A80192FD7847927||ACLTOP|||||LIS||P|1394-97|20160309090419
    P|1||||^|||U|||||^
    O|1|DDI CONTROLE L2||^^^2271|R|20160309085937|||||Q||||P||||||||||O@F
    R|1|^^^2271|1789|ng/mL||N||F@V||sh^sh||20160309090419|ACL TOP^DA^4
    C|1|I|LI-DDI CONTROLE L2^B25779^2017-4|I
    C|2|I|LI-DDi Latex^B25642^2017-2|I
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "B25779",
                "control_level": None,
                "control_name": None,
                "control_lot": "B25779",
                "results": [
                    {
                        "date": datetime.datetime(2016, 3, 9, 9, 4, 19),
                        "flags": ["N"],
                        "lots": [
                            {
                                "expiration": datetime.datetime(2017, 4, 1, 0, 0),
                                "lot": "DDI CONTROLE L2B25779",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2017, 2, 1, 0, 0),
                                "lot": "B25642",
                                "type": "control",
                            },
                            {
                                "expiration": datetime.datetime(2017, 2, 1, 0, 0),
                                "lot": "DDi LatexB25642",
                                "type": "reagent",
                            },
                        ],
                        "value": "1789",
                        "code": "2271ng/mL",
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_qc_reference_concatenee(self):
    self.configure(qc_reference_concatenee=True)
    result = self.decode(
        """
    H|@^\\|748E54D85F354FDF9A80192FD7847927||ACLTOP|||||LIS||P|1394-97|20160309090419
    P|1||||^|||U|||||^
    O|1|DDI CONTROLE L2||^^^2271|R|20160309085937|||||Q||||P||||||||||O@F
    R|1|^^^2271|1789|ng/mL||N||F@V||sh^sh||20160309090419|ACL TOP^DA^4
    C|1|I|5600^Maintenance dépassée ou en échec^MT|I
    C|2|I|LI-DDI CONTROLE L2^B25779^2017-4|I
    C|3|I|LI-DDi Latex^B25642^2017-2|I
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "DDI CONTROLE L2-B25779",
                "control_level": None,
                "control_name": None,
                "control_lot": "DDI CONTROLE L2-B25779",
                "results": [
                    {
                        "date": datetime.datetime(2016, 3, 9, 9, 4, 19),
                        "lots": [
                            {
                                "expiration": datetime.datetime(2017, 4, 1, 0, 0),
                                "lot": "DDI CONTROLE L2B25779",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2017, 2, 1, 0, 0),
                                "lot": "B25642",
                                "type": "control",
                            },
                            {
                                "expiration": datetime.datetime(2017, 2, 1, 0, 0),
                                "lot": "DDi LatexB25642",
                                "type": "reagent",
                            },
                        ],
                        "value": "1789",
                        "code": "2271ng/mL",
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_qc_version_6(self):
    self.configure(proto_version_6=True)
    result = self.decode(
        """
    H|@^\\|6B1662CE2EB54C4AA2C88BA4E9E80AD6||ACLTOP|||||LIS||P|1394-97|20160525113506
    P|1||||^|||U|||||^
    O|1|CQ Anormal Bas||^^^921|R|20160525083858|||||Q||||P||||||||||O@F
    R|1|^^^921|43.2|s||N||F@V||sh^sh||20160525084459|ACL TOP^DA^3
    C|1|I|LI-CQ Anormal Bas^N0355646^2018-3|I
    C|2|I|LI-TCA-SynthasIL CaCl2^N0757498^2018-7|I
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "CQ Anormal Bas",
                "control_level": "N0355646",
                "control_name": "CQ Anormal Bas",
                "control_lot": "CQ Anormal Bas",
                "results": [
                    {
                        "date": datetime.datetime(2016, 5, 25, 8, 44, 59),
                        "lots": [
                            {
                                "expiration": datetime.datetime(2018, 3, 1, 0, 0),
                                "lot": "CQ Anormal BasN0355646",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2018, 7, 1, 0, 0),
                                "lot": "N0757498",
                                "type": "control",
                            },
                            {
                                "expiration": datetime.datetime(2018, 7, 1, 0, 0),
                                "lot": "TCA-SynthasIL CaCl2N0757498",
                                "type": "reagent",
                            },
                        ],
                        "value": "43.2",
                        "code": "921s",
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_option_preliminary_results(self):
    self.configure(acltop_preliminary_results=True)
    result = self.decode(
        """
    H|@^\\|8CDBC50DDC0F4766A3F82421D72A9037||ACL TOP|||||LIS||P|1394-97|20230626172735
    P|1||14||Z-GILLE^Anne-sophie||19891128|F|||||^
    O|1|101010000088|<1687787579_826>|^^^131|R|20230626121600|||||||||P||||||||||O@P
    R|1|^^^131|Echec|s||N||P@X@V||acltopia^acltopia||20230626172238|ACL TOP^60^1
    C|1|I|5060^Delta minimum courbe non trouvée (Données)^CE|I
    C|2|I|LI-TP RecombiPlasTin^N0723781^2024-7|I
    C|3|I|LI-CQ Normal^N0723952^2025-7|I
    C|4|I|LI-CQ Anormal Bas^N0724169^2025-7|I
    C|5|I|LI-Calibration Plasma^E0824713^2025-8|I
    C|6|I|5100^Résultat mesuré en échec^RE|N
    R|2|^^^131|Echec|%||N||P@X@V||acltopia^acltopia||20230626172238|ACL TOP^60^1
    C|1|I|5060^Delta minimum courbe non trouvée (Données)^CE|I
    C|2|I|5100^Résultat mesuré en échec^RE|N
    R|3|^^^131|Echec|INR||N||P@X@V||acltopia^acltopia||20230626172238|ACL TOP^60^1
    C|1|I|5060^Delta minimum courbe non trouvée (Données)^CE|I
    C|2|I|5100^Résultat mesuré en échec^RE|N
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "131s",
                        "date": datetime.datetime(2023, 6, 26, 17, 22, 38),
                        "flags": ["5060", "CE", "N"],
                        "lots": [
                            {
                                "expiration": datetime.datetime(2024, 7, 1, 0, 0),
                                "lot": "TP RecombiPlasTinN0723781",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2025, 7, 1, 0, 0),
                                "lot": "CQ NormalN0723952",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2025, 7, 1, 0, 0),
                                "lot": "CQ Anormal BasN0724169",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2025, 8, 1, 0, 0),
                                "lot": "Calibration PlasmaE0824713",
                                "type": "reagent",
                            },
                        ],
                        "operator": "acltopia",
                        "value": "Echec",
                    },
                    {
                        "code": "131%",
                        "date": datetime.datetime(2023, 6, 26, 17, 22, 38),
                        "flags": ["5060", "CE", "N"],
                        "lots": [
                            {
                                "expiration": datetime.datetime(2024, 7, 1, 0, 0),
                                "lot": "TP RecombiPlasTinN0723781",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2025, 7, 1, 0, 0),
                                "lot": "CQ NormalN0723952",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2025, 7, 1, 0, 0),
                                "lot": "CQ Anormal BasN0724169",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2025, 8, 1, 0, 0),
                                "lot": "Calibration PlasmaE0824713",
                                "type": "reagent",
                            },
                        ],
                        "operator": "acltopia",
                        "value": "Echec",
                    },
                    {
                        "code": "131INR",
                        "date": datetime.datetime(2023, 6, 26, 17, 22, 38),
                        "flags": ["5060", "CE", "N"],
                        "lots": [
                            {
                                "expiration": datetime.datetime(2024, 7, 1, 0, 0),
                                "lot": "TP RecombiPlasTinN0723781",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2025, 7, 1, 0, 0),
                                "lot": "CQ NormalN0723952",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2025, 7, 1, 0, 0),
                                "lot": "CQ Anormal BasN0724169",
                                "type": "reagent",
                            },
                            {
                                "expiration": datetime.datetime(2025, 8, 1, 0, 0),
                                "lot": "Calibration PlasmaE0824713",
                                "type": "reagent",
                            },
                        ],
                        "operator": "acltopia",
                        "value": "Echec",
                    },
                ],
                "sample_id": "101010000088",
                "type": "patient",
            }
        ],
    }
    assert result == expected
