# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "clinitek"


def test_decode_result_production(self):
    worklist = self.decode(
        """
    MSH|^~\\&|Clinitek Status||||201802161451||ORU^R01|636543887330000005|P|2.4
    PID|1||||TEST
    PV1|1
    OBR|1||48|Multistix 10 SG|||||||||||UR^^^^^^P|||||705003|2018-10
    OBX|1|ST|GLU||Negatif||||||F||||||||201802161449
    OBX|2|ST|BIL||Negatif||||||F||||||||201802161449
    OBX|3|ST|KET||Negatif||||||F||||||||201802161449
    OBX|4|ST|SG||1.020||||||F||||||||201802161449
    OBX|5|ST|BLO||Negatif||||||F||||||||201802161449
    OBX|6|ST|pH||7.0||||||F||||||||201802161449
    OBX|7|ST|PRO||Negatif||||||F||||||||201802161449
    OBX|8|ST|URO||0.2|mg/dL|||||F||||||||201802161449
    OBX|9|ST|NIT||Negatif||||||F||||||||201802161449
    OBX|10|ST|LEU||Negatif||||||F||||||||201802161449
    OBX|11|ST|COL||Non saisie||||||F||||||||201802161449
    OBX|12|ST|CLA||Non saisie||||||F||||||||201802161449
    """
    )
    date = datetime.datetime(2018, 2, 16, 14, 49)
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "TEST",
                "results": [
                    {
                        "code": "GLU",
                        "value": "Negatif",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "",
                    },
                    {
                        "code": "BIL",
                        "value": "Negatif",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "",
                    },
                    {
                        "code": "KET",
                        "value": "Negatif",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "",
                    },
                    {
                        "code": "SG",
                        "value": "1.020",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "",
                    },
                    {
                        "code": "BLO",
                        "value": "Negatif",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "",
                    },
                    {
                        "code": "pH",
                        "value": "7.0",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "",
                    },
                    {
                        "code": "PRO",
                        "value": "Negatif",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "",
                    },
                    {
                        "code": "URO",
                        "value": "0.2",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "",
                    },
                    {
                        "code": "NIT",
                        "value": "Negatif",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "",
                    },
                    {
                        "code": "LEU",
                        "value": "Negatif",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "",
                    },
                    {
                        "code": "COL",
                        "value": "Non saisie",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "",
                    },
                    {
                        "code": "CLA",
                        "value": "Non saisie",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_example(self):
    worklist = self.decode(
        """
    MSH|^~\\&|Clinitek Status||||201701050228||ORU^R01|630826342120170001|P|2.4
    PID|1||XWN||ABR
    NTE|1|L|DTitle:CUST|G
    NTE|2|L|DValue:POM|G
    PV1|1
    OBR|1||2|Multistix 10 SG|||||||||||UR^^^^^^P|||||GFP
    OBX|1|ST|GLU||Negative||||||F|||||HOV|||201701050226
    OBX|2|ST|BIL||Negative||||||F|||||HOV|||201701050226
    OBX|3|ST|KET||Negative||||||F|||||HOV|||201701050226
    OBX|4|ST|SG||<=1.005||||||F|||||HOV|||201701050226
    OBX|5|ST|BLO||Large|||A|||F|||||HOV|||201701050226
    OBX|6|ST|pH||6.5||||||F|||||HOV|||201701050226
    OBX|7|ST|PRO||>=300|mg/dL||A|||F|||||HOV|||201701050226
    NTE|2|L|CRSNote1:Visibly bloody urine may cause falsely elevated PRO results|G
    OBX|8|ST|URO||0.2|E.U./dL|||||F|||||HOV|||201701050226
    OBX|9|ST|NIT||Negative||||||F|||||HOV|||201701050226
    OBX|10|ST|LEU||Negative||||||F|||||HOV|||201701050226
    OBX|11|ST|COL||Yellow||||||F|||||HOV|||201701050226
    OBX|12|ST|CLA||Clear||||||F|||||HOV|||201701050226
    """
    )
    date = datetime.datetime(2017, 1, 5, 2, 26)
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "ABR",
                "results": [
                    {
                        "code": "GLU",
                        "value": "Negative",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "HOV",
                    },
                    {
                        "code": "BIL",
                        "value": "Negative",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "HOV",
                    },
                    {
                        "code": "KET",
                        "value": "Negative",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "HOV",
                    },
                    {
                        "code": "SG",
                        "value": "<=1.005",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "HOV",
                    },
                    {
                        "code": "BLO",
                        "value": "Large",
                        "flags": ["A"],
                        "comments": [],
                        "date": date,
                        "operator": "HOV",
                    },
                    {
                        "code": "pH",
                        "value": "6.5",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "HOV",
                    },
                    {
                        "code": "PRO",
                        "value": ">=300",
                        "flags": ["A"],
                        "comments": [
                            "Visibly bloody urine may cause falsely elevated PRO results"
                        ],
                        "date": date,
                        "operator": "HOV",
                    },
                    {
                        "code": "URO",
                        "value": "0.2",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "HOV",
                    },
                    {
                        "code": "NIT",
                        "value": "Negative",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "HOV",
                    },
                    {
                        "code": "LEU",
                        "value": "Negative",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "HOV",
                    },
                    {
                        "code": "COL",
                        "value": "Yellow",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "HOV",
                    },
                    {
                        "code": "CLA",
                        "value": "Clear",
                        "flags": [],
                        "comments": [],
                        "date": date,
                        "operator": "HOV",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    worklist = self.decode(
        """
    MSH|^~\\&|Clinitek Status||||201701050236||ORU^R01|630826342120170003|P|2.4
    OBR|1||4|Multistix PRO 10LS|||||||||||ORH^^^^^^Q|||||STRIP
    NTE|1|L|QCName:STUFF|G
    NTE|2|L|QCLevel:PON|G
    NTE|3|L|QCLot:CLOT|G
    NTE|4|L|QCExpiry:2017-01-05|G
    NTE|5|L|QCResult:PASS|G
    NTE|6|L|QCSuite:43|G
    OBX|1|ST|GLU||Negative||||||F|||||HOL|||201701050235
    NTE|1|L|CCRResult:|G
    OBX|2|ST|KET||Negative||||||F|||||HOL|||201701050235
    NTE|1|L|CCRResult:|G
    OBX|3|ST|SG||1.010||||||F|||||HOL|||201701050235
    NTE|1|L|CCRResult:|G
    OBX|4|ST|pH||6.5||||||F|||||HOL|||201701050235
    NTE|1|L|CCRResult:|G
    OBX|5|ST|BLO||Large||||||F|||||HOL|||201701050235
    NTE|1|L|CCRResult:|G
    OBX|6|ST|NIT||Negative||||||F|||||HOL|||201701050235
    NTE|1|L|CCRResult:|G
    """
    )
    date = datetime.datetime(2017, 1, 5, 2, 35)
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "control_lot": "CLOT",
                "sample_id": "CLOT",
                "results": [
                    {
                        "code": "GLU",
                        "value": "Negative",
                        "flags": [],
                        "date": date,
                        "operator": "HOL",
                    },
                    {
                        "code": "KET",
                        "value": "Negative",
                        "flags": [],
                        "date": date,
                        "operator": "HOL",
                    },
                    {
                        "code": "SG",
                        "value": "1.010",
                        "flags": [],
                        "date": date,
                        "operator": "HOL",
                    },
                    {
                        "code": "pH",
                        "value": "6.5",
                        "flags": [],
                        "date": date,
                        "operator": "HOL",
                    },
                    {
                        "code": "BLO",
                        "value": "Large",
                        "flags": [],
                        "date": date,
                        "operator": "HOL",
                    },
                    {
                        "code": "NIT",
                        "value": "Negative",
                        "flags": [],
                        "date": date,
                        "operator": "HOL",
                    },
                ],
            }
        ],
    }
    assert worklist == expected
