# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import Worklist

parser_class = "elo_hl7"


def test_encode_order(self):
    worklist = Worklist(
        context="order",
        samples=[
            {
                "sample_id": "0102030405",
                "type": "patient",
                "last_name": "Z_LABO",
                "first_name": "Tasty",
                "title": "Mme",
                "sex": "F",
                "patient_id": "0102030405",
                "birthdate": datetime.datetime(1990, 1, 28),
                "request_date": datetime.datetime(2018, 7, 31, 15, 10, 24),
                "specimen": "S",
                "orders": [
                    {"code": "ANA", "label": "Cell nuclei (ANA)", "action": "add"}
                ],
                "physicians": [
                    {
                        "code": "PRESCRIBER_ID",
                        "last_name": "Last Name",
                        "first_name": "First Name",
                        "type": "prescriber",
                    }
                ],
            }
        ],
    )
    message = self.encode(
        worklist,
        context={
            "now": datetime.datetime(2018, 7, 31, 15, 10, 24),
            "control_id": "9514753",
        },
    )
    expected = """
    MSH|^~\\&|MCA|MCA|||20180731151024||ORM^001|9514753|P|2.4|||||FRA||FR
    PID|0102030405|0102030405|||Z_LABO^Tasty||19900128|F
    ORC|NW||||||||20180731151024|||PRESCRIBER_ID^Last Name^First Name
    OBR|1|0102030405|||R||20180731151024||||||||S
    OBX|1||ANA^Cell nuclei (ANA)||||||||I|||20180731151024
    """
    assert message == expected


def test_encode_order_with_external_results(self):
    worklist = Worklist(
        context="order",
        samples=[
            {
                "sample_id": "0102030405",
                "type": "patient",
                "last_name": "Z_LABO",
                "first_name": "Tasty",
                "title": "Mme",
                "sex": "F",
                "patient_id": "0102030405",
                "birthdate": datetime.datetime(1990, 1, 28),
                "request_date": datetime.datetime(2018, 7, 31, 15, 10, 24),
                "specimen": "S",
                "orders": [
                    {"code": "ANA", "label": "Cell nuclei (ANA)", "action": "add"}
                ],
                "physicians": [
                    {
                        "code": "PRESCRIBER_ID",
                        "last_name": "Last Name",
                        "first_name": "First Name",
                        "type": "prescriber",
                    }
                ],
                "results": [
                    {
                        "type": "SN",
                        "code": "B2GPG",
                        "label": "Beta2 glycoprotein IgG",
                        "type": "SN",
                        "value": "5",
                        "unit": "U/mL",
                        "reference_ranges": "0,0 - 10,0",
                        "flags": ["N"],
                        "date": datetime.datetime(2011, 5, 6, 12, 13),
                    }
                ],
            }
        ],
    )
    message = self.encode(
        worklist,
        context={
            "now": datetime.datetime(2018, 7, 31, 15, 10, 24),
            "control_id": "9514753",
        },
    )
    expected = """
    MSH|^~\\&|MCA|MCA|||20180731151024||ORM^001|9514753|P|2.4|||||FRA||FR
    PID|0102030405|0102030405|||Z_LABO^Tasty||19900128|F
    ORC|NW||||||||20180731151024|||PRESCRIBER_ID^Last Name^First Name
    OBR|1|0102030405|||R||20180731151024||||||||S
    OBX|1||ANA^Cell nuclei (ANA)||||||||I|||20180731151024
    ORC|XE||||||||20180731151024|||PRESCRIBER_ID^Last Name^First Name
    OBR|1|0102030405|||R||20180731151024||||||||S
    OBX|1|SN|B2GPG^Beta2 glycoprotein IgG||5|U/mL|0,0 - 10,0|N|||P|20110506121300||20180731151024
    """
    assert message == expected


def test_decode_result(self):
    date = datetime.datetime(2012, 12, 3, 8, 12, 20)
    worklist = self.decode(
        """
    MSH|^~\\&|EUROLabOffice|EUROIMMUN|||20121203081532||ORU^R01|263|P|2.4
    PID|||||last name1^first name1||19101010|M
    ORC|SC|123456|||||||20121203081533|||Sender|||||||EUROLabOffice
    OBR||123456|||||20121203081533|||||||||Sender
    OBX||ST|ANAIFA^Cell nuclei (ANA)||positive||||||P|20121203081220||20121203081220
    OBX||SN|ANATit^Cell nuclei (ANA)||160|Cut off|<1:100|N|||P|20121203081220||20121203081220
    OBX||ST|ANAPat^Cell nuclei (ANA)||homogenous||||||P|20121203081220||20121203081220
    OBX||SN|ANAT2^Cell nuclei (ANA)||320|Cut off|<1:100||||P|20121203081220||20121203081220
    OBX||ST|ANAP2^Cell nuclei (ANA)||speckled||||||P|20121203081220||20121203081220
    OBX||ST|ANAComment^Cell nuclei (ANA)||Official Remark||||||P|20121203081220||20121203081220
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "ANAIFA", "date": date, "flags": [], "value": "positive"},
                    {
                        "code": "ANATit",
                        "date": date,
                        "flags": ["N"],
                        "unit": "Cut off",
                        "value": "160",
                    },
                    {
                        "code": "ANAPat",
                        "date": date,
                        "flags": [],
                        "value": "homogenous",
                    },
                    {
                        "code": "ANAT2",
                        "date": date,
                        "flags": [],
                        "unit": "Cut off",
                        "value": "320",
                    },
                    {"code": "ANAP2", "date": date, "flags": [], "value": "speckled"},
                    {
                        "code": "ANAComment",
                        "date": date,
                        "flags": [],
                        "value": "Official Remark",
                    },
                ],
                "sample_id": "123456",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    self.configure(elo_hl7_qc_prefix="QC")
    date = datetime.datetime(2012, 12, 3, 8, 12, 24)
    worklist = self.decode(
        """
    MSH|^~\\&|EUROLabOffice|EUROIMMUN|||20121203085950||ORU^R01|264|P|2.4
    PID|||||last name2^first name2||19111111|F
    ORC|SC|987654|||||||20121203085950|||Sender|||||||EUROLabOffice
    OBR||QC987654|||||20121203085950|||||||||Sender
    OBX||ST|Co^EUROLINE ANA Profile 3||positive||||||P|20121203081224||20121203081224
    OBX||ST|ACM2^EUROLINE ANA Profile 3||negative||||||P|20121203081224||20121203081224
    OBX||ST|RIB^EUROLINE ANA Profile 3||positive||||||P|20121203081224||20121203081224
    OBX||ST|HI^EUROLINE ANA Profile 3||positive||||||P|20121203081224||20121203081224
    OBX||ST|NUC^EUROLINE ANA Profile 3||negative||||||P|20121203081224||20121203081224
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "QC987654",
                "results": [
                    {"code": "Co", "date": date, "flags": [], "value": "positive"},
                    {"code": "ACM2", "date": date, "flags": [], "value": "negative"},
                    {"code": "RIB", "date": date, "flags": [], "value": "positive"},
                    {"code": "HI", "date": date, "flags": [], "value": "positive"},
                    {"code": "NUC", "date": date, "flags": [], "value": "negative"},
                ],
                "sample_id": "QC987654",
                "type": "control",
            }
        ],
    }
    assert worklist == expected
