# -*- coding: utf8 -*
from __future__ import unicode_literals

import datetime

parser_class = "id_now"


def test_decode_result(self):
    date = datetime.datetime(2016, 6, 20, 14, 46, 22)
    worklist = self.decode(
        """
    H|\\^&|||ALERE.ALEREI^4.4.0.6.126-pro^A6A03616|||||||P|1|20160620175200
    P|1||10X1234
    O|1||a3fca28e-bade-4117-85de-a27a9d16dc5d|^^^Influenza A &E& B|||||||||||Vtm^^2122430||||||||||F
    R|1|^^^Flu A^M|Negative|||N||F||staff|20160620143618|20160620144622|A6A03616
    R|2|^^^Flu B^M|Positive|||N||F|||||A6A03616
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "10X1234",
                "specimen": "Vtm",
                "results": [
                    {
                        "code": "Flu A",
                        "date": date,
                        "value": "Negative",
                        "flags": ["N"],
                        "operator": "staff",
                        "instrument": "A6A03616",
                    },
                    {
                        "code": "Flu B",
                        "date": date,
                        "value": "Positive",
                        "flags": ["N"],
                        "operator": "staff",
                        "instrument": "A6A03616",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    date = datetime.datetime(2013, 2, 27, 9, 18, 32)
    worklist = self.decode(
        """
    H|\\^&|||ALERE.ALEREI^4.4.0.6.126-pro^8FC07715|||||||P|1|20160606174026
    P|1
    O|1||73d80789-48a7-45a9-b536-45d6420f7f89|^^^Streptococcus A|||||||||||QC^^BD003636^^QC Positive||||||||||F
    R|1|^^^Strep A^M|Positive|||N||F||19283741|20130227091126|20130227091832|8FC07715
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "specimen": "QC",
                "control_lot": "BD003636",
                "sample_id": "QC Positive",
                "results": [
                    {
                        "code": "Strep A",
                        "date": date,
                        "value": "Positive",
                        "flags": ["N"],
                        "operator": "19283741",
                        "instrument": "8FC07715",
                    }
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_double_control(self):
    date = datetime.datetime(2013, 2, 27, 14, 36, 30)
    worklist = self.decode(
        """
    H|\\^&|||ALERE.ALEREI^4.4.0.6.126-pro^8FC07715|||||||P|1|20160606172748
    P|1
    O|1||21fb91e4-1bf7-4782-b795-6128a9e4aaaa|^^^Influenza A &E& B|||||||||||QC^^c4g56h2b78e5^^QC Positive||||||||||F
    R|1|^^^Flu A^M|Negative|||A||F||chawkins|20130227142600|20130227143630|8FC07715
    R|2|^^^Flu B^M|Negative|||A||F|||||8FC07715
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "specimen": "QC",
                "control_lot": "c4g56h2b78e5",
                "sample_id": "QC Positive",
                "results": [
                    {
                        "code": "Flu A",
                        "date": date,
                        "value": "Negative",
                        "flags": ["A"],
                        "operator": "chawkins",
                        "instrument": "8FC07715",
                    },
                    {
                        "code": "Flu B",
                        "date": date,
                        "value": "Negative",
                        "flags": ["A"],
                        "operator": "chawkins",
                        "instrument": "8FC07715",
                    },
                ],
            }
        ],
    }
    assert worklist == expected
