# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "scanbac"


def test_decode_order(self):
    worklist = self.decode(
        """
    H|~^\\&|||000~ZZZ||ORM|||111~XXX||P|A.2.|20150615090126
    P|0001|67290|||ANO~NYME~~~~||19360901|F|
    C|1|L|AAAA|
    C|1|L|BBBB|
    OBR|0001|~50620150621||CBU2~~L~|R~N||||||R|||||PRS~NAME||PREOP|
    OBX|1|FIC|CBU2|1|~1434351721656887-5062015062.TIF||
    OBX|2|FIC|CBU2|2|~1434351721657059-5062015062.TIF||
    OBR|0002|~50620150622||BACA~~L~|R~N||||||R|||||PRS~NAME||PREOP|
    OBX|1|FIC|BACA|1|~1434351721784591-5062015062.TIF||
    OBX|2|FIC|BACA|2|~1434351721784696-5062015062.TIF||
    L|1||1|11|"""
    )
    date = datetime.datetime(2015, 6, 15, 9, 1, 26)
    expected = {
        "context": "order",
        "sender_id": "CERDISCAN",
        "receiver_id": "MCA",
        "samples": [
            {
                "type": "patient",
                "sample_id": "50620150621",
                "request_id": "50620150621",
                "patient_id": "67290",
                "last_name": "ANO",
                "first_name": "NYME",
                "birthdate": datetime.datetime(1936, 9, 1, 0, 0),
                "request_date": date,
                "collection_date": date,
                "comments": [
                    {"text": "AAAA", "type": "patient"},
                    {"text": "BBBB", "type": "patient"},
                ],
                "physicians": [
                    {"code": "PRS", "last_name": "NAME", "type": "prescriber"}
                ],
                "orders": [{"action": "add", "code": "CBU2", "priority": "R"}],
                "results": [
                    {
                        "anterior": False,
                        "code": "CBU2",
                        "sub_code": "1",
                        "value": "1434351721656887-5062015062.TIF",
                    },
                    {
                        "anterior": False,
                        "code": "CBU2",
                        "sub_code": "2",
                        "value": "1434351721657059-5062015062.TIF",
                    },
                ],
                "sex": "F",
            },
            {
                "type": "patient",
                "sample_id": "50620150622",
                "request_id": "50620150622",
                "patient_id": "67290",
                "last_name": "ANO",
                "first_name": "NYME",
                "birthdate": datetime.datetime(1936, 9, 1, 0, 0),
                "collection_date": date,
                "comments": [
                    {"text": "AAAA", "type": "patient"},
                    {"text": "BBBB", "type": "patient"},
                ],
                "physicians": [
                    {"code": "PRS", "last_name": "NAME", "type": "prescriber"}
                ],
                "request_date": date,
                "orders": [{"action": "add", "code": "BACA", "priority": "R"}],
                "results": [
                    {
                        "anterior": False,
                        "code": "BACA",
                        "sub_code": "1",
                        "value": "1434351721784591-5062015062.TIF",
                    },
                    {
                        "anterior": False,
                        "code": "BACA",
                        "sub_code": "2",
                        "value": "1434351721784696-5062015062.TIF",
                    },
                ],
                "sex": "F",
            },
        ],
    }
    assert worklist == expected
